﻿using KellermanSoftware.NetSFtpLibrary;
using System;
using System.IO;

namespace KellermanSoftware.NetSFtpLibraryCoreDemo
{
    internal sealed class SftpClient : IDisposable
    {
        private readonly SFTP _sftp;

        public SftpClient(Configuration configuration)
        {
            _sftp = new SFTP(configuration.LicenseUsername, configuration.LicenseKey)
            {
                HostAddress = configuration.Server,
                UserName = configuration.Username,
                Password = configuration.Password
            };
        }

        public void Dispose()
        {
            if (_sftp != null)
            {
                _sftp.Dispose();
            }
        }

        public void Upload(string source, string destination)
        {
            if (File.Exists(source))
            {
                UploadFile(source, destination);
            }
            else if (Directory.Exists(source))
            {
                UploadDirectory(source, destination);
            }
            else
            {
                throw new FileNotFoundException($"Cannot find source '{source}'");
            }
        }

        public void Download(string source, string destination)
        {
            try
            {
                _sftp.Connect();
                if (_sftp.DirectoryExists(source))
                {
                    _sftp.DownloadDirectory(destination, source);
                }
                else if (_sftp.FileExists(source))
                {
                    _sftp.DownloadFile(destination, source);
                }
                else
                {
                    throw new FileNotFoundException($"Cannot find source '{source}'");
                }
            }
            finally
            {
                _sftp.Disconnect();
            }
        }

        private void UploadFile(string sourceFile, string destinationFile)
        {
            try
            {
                _sftp.Connect();
                _sftp.UploadFile(sourceFile, destinationFile);
            }
            finally
            {
                _sftp.Disconnect();
            }
        }

        private void UploadDirectory(string sourceDirectory, string destinationDirectory)
        {
            try
            {
                _sftp.Connect();
                _sftp.UploadDirectory(sourceDirectory, destinationDirectory);
            }
            finally
            {
                _sftp.Disconnect();
            }
        }
    }
}
